@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +15,6 @@
 #include <libqalculate/includes.h>
 #include <libqalculate/util.h>
 #include <sys/time.h>
-#include <pthread.h>
 
 /** @file */
 
@@ -33,14 +32,16 @@
 * CALCULATOR->loadGlobalDefinitions();
 * CALCULATOR->loadLocalDefinitions();
 * EvaluationOptions eo;
-* MathStructure result = CALCULATOR->calculate("1 + 1", eo);\endcode
+* MathStructure result;
+* CALCULATOR->calculate(&mstruct, "1 + 1", 2000, eo);\endcode
+* In the above example, the calculation is terminated after two seconds (2000 ms), if it is not finished before then.
 *
 * More complex usage mainly involves manipulating objects of the MathStructure class directly.
 *
-* To display the resulting expression you will normally use MathStructure::format() followed by MathStructure::print() as follows:
+* To display the resulting expression use Calculator::print() as follows:
 * \code PrintOptions po;
-* result.format(po);
-* string result_str = result.print(po); \endcode
+* string result_str = CALCULATOR->print(result, 2000, po);\endcode
+* Alternatively MathStructure::format() followed by MathStructure::print() can be used, whithout the possiblity to specify a time limit.
 *
 * Central to the flexiblity of libqalculate is the many options passed to evaluating and display functions with EvaluationOptions and PrintOptions.
 *
@@ -51,9 +52,9 @@
 * For a simple program use pkg-config on the command line:
 * \code c++ `pkg-config --cflags --libs libqalculate` hello.c -o hello \endcode
 *
-* If the program uses autoconf, put the following in configure.in:
+* If the program uses autoconf, put the following in configure.ac:
 * \code PKG_CHECK_MODULES(QALCULATE, [
-*	libqalculate >= 0.9.11
+*	libqalculate >= 1.0.0
 *	])
 * AC_SUBST(QALCULATE_CFLAGS)
 * AC_SUBST(QALCULATE_LIBS) \endcode
@@ -227,9 +228,9 @@ class Calculator {
 	vector<int> stopped_errors_count;
 	vector<int> stopped_warnings_count;
 	vector<int> stopped_messages_count;
-	pthread_t calculate_thread;
-	pthread_attr_t calculate_thread_attr;
-	bool b_functions_was, b_variables_was, b_units_was, b_unknown_was, b_calcvars_was, b_rpn_was;
+
+	Thread *calculate_thread;
+
 	string NAME_NUMBER_PRE_S, NAME_NUMBER_PRE_STR, DOT_STR, DOT_S, COMMA_S, COMMA_STR, ILLEGAL_IN_NAMES, ILLEGAL_IN_UNITNAMES, ILLEGAL_IN_NAMES_MINUS_SPACE_STR;
 
 	bool b_argument_errors;
@@ -238,38 +239,36 @@ class Calculator {
 	time_t exchange_rates_time, exchange_rates_check_time;
 	bool b_exchange_rates_used, b_exchange_rates_warning_enabled;
 
-	int has_gvfs;
-
 	bool b_gnuplot_open;
 	string gnuplot_cmdline;
-	FILE *gnuplot_pipe, *calculate_pipe_r, *calculate_pipe_w;
+	FILE *gnuplot_pipe;
 	
 	bool local_to;
 	
 	Assumptions *default_assumptions;
 	
 	vector<Variable*> deleted_variables;
-	vector<MathFunction*> deleted_functions;	
+	vector<MathFunction*> deleted_functions;
 	vector<Unit*> deleted_units;
 	
 	bool b_save_called;
 	
-	int i_print_timeout;
-	struct timeval t_print_end;
-	int i_printing_aborted;
-	bool b_printing_controlled;
+	int i_timeout;
+	struct timeval t_end;
+	int i_aborted;
+	bool b_controlled;
 	
 	string per_str, times_str, plus_str, minus_str, and_str, AND_str, or_str, OR_str, XOR_str;
 	size_t per_str_len, times_str_len, plus_str_len, minus_str_len, and_str_len, AND_str_len, or_str_len, OR_str_len, XOR_str_len;
 
 	vector<MathStructure*> rpn_stack;
 
-	bool calculateRPN(MathStructure *mstruct, int command, size_t index, int msecs, const EvaluationOptions &eo);
-	bool calculateRPN(string str, int command, size_t index, int msecs, const EvaluationOptions &eo, MathStructure *parsed_struct, MathStructure *to_struct, bool make_to_division);
+	bool calculateRPN(MathStructure *mstruct, int command, size_t index, int msecs, const EvaluationOptions &eo, int function_arguments = 0);
+	bool calculateRPN(string str, int command, size_t index, int msecs, const EvaluationOptions &eo, MathStructure *parsed_struct, MathStructure *to_struct, bool make_to_division, int function_arguments = 0);
 	
   public:
 
-	KnownVariable *v_pi, *v_e, *v_i, *v_inf, *v_pinf, *v_minf, *v_undef;
+	KnownVariable *v_pi, *v_e, *v_euler, *v_catalan, *v_i, *v_inf, *v_pinf, *v_minf, *v_undef, *v_precision;
 	UnknownVariable *v_x, *v_y, *v_z;
 	MathFunction *f_vector, *f_sort, *f_rank, *f_limits, *f_component, *f_dimension, *f_merge_vectors;
 	MathFunction *f_matrix, *f_matrix_to_vector, *f_area, *f_rows, *f_columns, *f_row, *f_column, *f_elements, *f_element, *f_transpose, *f_identity, *f_determinant, *f_permanent, *f_adjoint, *f_cofactor, *f_inverse; 
@@ -282,8 +281,8 @@ class Calculator {
 	MathFunction *f_exp;
 	MathFunction *f_ln, *f_logn;
 	MathFunction *f_lambert_w;
-	MathFunction *f_sin, *f_cos, *f_tan, *f_asin, *f_acos, *f_atan, *f_sinh, *f_cosh, *f_tanh, *f_asinh, *f_acosh, *f_atanh, *f_radians_to_default_angle_unit;
-	MathFunction *f_zeta, *f_gamma, *f_beta;
+	MathFunction *f_sin, *f_cos, *f_tan, *f_asin, *f_acos, *f_atan, *f_sinh, *f_cosh, *f_tanh, *f_asinh, *f_acosh, *f_atanh, *f_atan2, *f_radians_to_default_angle_unit;
+	MathFunction *f_zeta, *f_gamma, *f_digamma, *f_beta, *f_airy, *f_besselj, *f_bessely, *f_erf, *f_erfc;
 	MathFunction *f_total, *f_percentile, *f_min, *f_max, *f_mode, *f_rand;
 	MathFunction *f_isodate, *f_localdate, *f_timestamp, *f_stamptodate, *f_days, *f_yearfrac, *f_week, *f_weekday, *f_month, *f_day, *f_year, *f_yearday, *f_time, *f_add_days, *f_add_months, *f_add_years;
 	MathFunction *f_bin, *f_oct, *f_hex, *f_base, *f_roman;
@@ -297,7 +296,7 @@ class Calculator {
 	MathFunction *f_plot;
 	MathFunction *f_uncertainty;
 	
-	Unit *u_rad, *u_gra, *u_deg, *u_euro, *u_kelvin, *u_celsius, *u_fahrenheit;
+	Unit *u_rad, *u_gra, *u_deg, *u_euro;
 	DecimalPrefix *decimal_null_prefix;
 	BinaryPrefix *binary_null_prefix;
 
@@ -305,7 +304,7 @@ class Calculator {
 	bool place_currency_code_before_negative, place_currency_sign_before_negative;
 	bool default_dot_as_separator;
   
-  	bool b_busy, calculate_thread_stopped;
+  	bool b_busy;
 	string expression_to_calculate;
 	EvaluationOptions tmp_evaluationoptions;
 	MathStructure *tmp_parsedstruct;
@@ -313,6 +312,7 @@ class Calculator {
 	MathStructure *tmp_rpn_mstruct;
 	bool tmp_maketodivision;
 	int tmp_proc_command;
+	int tmp_proc_registers;
 	size_t tmp_rpnindex;
 	
 	PrintOptions save_printoptions;	
@@ -356,44 +356,76 @@ class Calculator {
 	* @returns The result of the calculation.
 	*/
 	MathStructure calculate(string str, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
+	/** Calculates a parsed value.
+	* This function starts the calculation in a separate thread and will return when the calculation has started unless a maximum time has been specified.
+	* The calculation can then be stopped with abort().
+	*
+	* @param[out] mstruct Parsed value to evaluate and fill with the result.
+	* @param msecs The maximum time for the calculation in milliseconds. If msecs <= 0 the time will be unlimited.
+	* @param eo Options for the evaluation of the expression.
+	* @param to_str "to" expression for conversion.
+	* @returns The result of the calculation.
+	*/
+	bool calculate(MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_evaluation_options, string to_str = "");
+	/** Calculates a parsed value.
+	*
+	* @param mstruct Parsed value to evaluate.
+	* @param eo Options for the evaluation of the expression.
+	* @param to_str "to" expression for conversion.
+	* @returns The result of the calculation.
+	*/
+	MathStructure calculate(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options, string to_str = "");
 	int testCondition(string expression);
 	//@}
 
-	/** @name Functions for printing expressions with the option to set a timeout or abort. */
+	/** @name Functions for printing expressions with the option to set maximum time or abort. */
 	//@{
 	/** Calls MathStructure::format(po) and MathStructure::print(po). The process is aborted after msecs milliseconds.
 	*/
-	string printMathStructureTimeOut(const MathStructure &mstruct, int msecs = 100000, const PrintOptions &op = default_print_options);
-	/** Called before formatting and printing a MathStructure (using MathStructure::format() and MathStructure::print()) 
-	* or a Number (using Number::print) to be able to abort the process. Always use Calculator::abortPrint() after finishing.
+	string print(const MathStructure &mstruct, int milliseconds = 100000, const PrintOptions &op = default_print_options);
+	///Deprecated: use print() instead
+	string printMathStructureTimeOut(const MathStructure &mstruct, int milliseconds = 100000, const PrintOptions &op = default_print_options);
+	
+	/** Called before calculation, formatting or printing of a MathStructure (Calculator::calculate(), without maximum time, MathStructure::eval(), MathStructure::format() and MathStructure::print(), etc.) or printing of a Number (using Number::print) to be able to abort the process. Always use Calculator::stopControl() after finishing.
 	*
-	* @param msecs The maximum time for formatting printing in milliseconds. If msecs <= 0 the time will be unlimited.
+	* @param milliseconds The maximum time for the process in milliseconds. If msecs <= 0 the time will be unlimited (stop with abort()).
 	*/
-	void startPrintControl(int milli_timeout = 0);
-	/** Abort formatting and printing (after startPrintControl() has been called).
-	* This function will normally be called from a thread that checks for used input or other conditions.
-	*/	
-	void abortPrint(void);
-	/** Always call this function after Calculator::startPrintControl() after formatting and printing has finished.
+	void startControl(int milliseconds = 0);
+	/** Always call this function after Calculator::startControl() after formatting, printing or calculation has finished.
 	*/
-	void stopPrintControl(void);
-	bool printingAborted(void);
-	bool printingControlled(void);
-	string printingAbortedMessage(void) const;
+	void stopControl(void);
+	/** Abort formatting, printing or evaluation (after startControl() has been called).
+	* This function will normally be called from a thread that checks for user input or other conditions.
+	*
+	* @returns false if the calculation thread was forcibly stopped
+	*/
+	bool abort();
+	bool aborted(void);
+	bool isControlled(void) const;
+	string abortedMessage(void) const;
 	string timedOutString(void) const;
+	
+	///Deprecated: use startControl()
+	void startPrintControl(int milliseconds = 0);
+	///Deprecated: use abort()
+	void abortPrint(void);
+	///Deprecated: use stopControl()
+	void stopPrintControl(void);
+	/// Deprecated: use aborted()
+	bool printingAborted(void);
+	/// Deprecated: use isControlled()
+	bool printingControlled(void) const;
+	/// Deprecated: use abortedMessage()
+	string printingAbortedMessage(void) const;
 	//@}
 
 	/** @name Functions for handling of threaded calculations */
 	//@{
-	/** Aborts the current calculation. */
-	void abort();
-	/** Aborts the current calculation. Used from within the calculation thread. */
-	void abort_this();
 	/** Returns true if the calculate or print thread is busy. */
 	bool busy();
-	/** Saves the state of the calculator. Used internally to be able to restore the state after aborted calculation. */
+	/// Deprecated: does nothing.
 	void saveState();
-	/** Restores the saved state of the calculator. Used internally to restore the state after aborted calculation. */
+	/// Deprecated: does nothing.
 	void restoreState();
 	/** Clears all stored values. Used internally after aborted calculation. */
 	void clearBuffers();
@@ -581,7 +613,7 @@ class Calculator {
 	void parse(MathStructure *mstruct, string str, const ParseOptions &po = default_parse_options);
 	bool parseNumber(MathStructure *mstruct, string str, const ParseOptions &po = default_parse_options);
 	bool parseOperators(MathStructure *mstruct, string str, const ParseOptions &po = default_parse_options);
-	bool parseAdd(string &str, MathStructure *mstruct, const ParseOptions &po, MathOperation s);
+	bool parseAdd(string &str, MathStructure *mstruct, const ParseOptions &po, MathOperation s, bool append = true);
 	bool parseAdd(string &str, MathStructure *mstruct, const ParseOptions &po);
 	//@}
 
@@ -593,9 +625,10 @@ class Calculator {
 	* @param mstruct The value to convert.
 	* @param composite_ Unit expression.
 	* @param eo Evaluation options.
+	* @param[out] units NULL or a math structure to fill with the parsed unit expression(or set to undefined if no units were found).
 	* @returns Converted value.
 	*/
-	MathStructure convert(const MathStructure &mstruct, string composite_, const EvaluationOptions &eo = default_evaluation_options);
+	MathStructure convert(const MathStructure &mstruct, string composite_, const EvaluationOptions &eo = default_evaluation_options, MathStructure *units = NULL);
 	/** Converts to a unit.
 	* The converted value is evaluated.
 	*
@@ -607,11 +640,13 @@ class Calculator {
 	*/
 	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options, bool always_convert = true, bool convert_to_mixed_units = true);
 	MathStructure convert(double value, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options);
-	MathStructure convertTimeOut(string str, Unit *from_unit, Unit *to_unit, int msecs, const EvaluationOptions &eo = default_evaluation_options);
+	MathStructure convert(string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_evaluation_options);
+	/** Depecated: use convert() */
+	MathStructure convertTimeOut(string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_evaluation_options);
 	MathStructure convert(string str, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options);	
 	MathStructure convertToBaseUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options);
 	Unit *getBestUnit(Unit *u, bool allow_only_div = false);
-	MathStructure convertToBestUnit(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options);
+	MathStructure convertToBestUnit(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options, bool convert_to_si_units = true);
 	MathStructure convertToCompositeUnit(const MathStructure &mstruct, CompositeUnit *cu, const EvaluationOptions &eo = default_evaluation_options, bool always_convert = true);
 	MathStructure convertToMixedUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options);
 	//@}
@@ -956,22 +991,14 @@ class Calculator {
 	//@}
 	
 	/** @name Functions for exchange rates. */
-	//@{
-	/** Checks if gvfs-copy or wget is available for downloading exchange rates from the Internet.
+	/** Checks if able to downloading exchange rates from the Internet (using libcurl).
 	*
-	* @returns true if gvfs-copy or wget was found.
+	* @returns true if exchange rates can downloaded (if libcurl is available).
 	*/
 	bool canFetch();
-	/** Checks if gvfs-copy available.
-	*
-	* @returns true if gvfs-copy was found.
-	*/
+	///Deprecated: gvfs is not needed anymore.
 	bool hasGVFS();
-	/** Checks if gvfs-copy available. Deprecated: use hasGVFS() or canFetch() instead.
-	*
-	*
-	* @returns true if gvfs-copy was found.
-	*/
+	///Deprecated: gvfs is not needed anymore.
 	bool hasGnomeVFS();
 	/** Load saved (local) currency units and exchange rates.
 	*
@@ -993,19 +1020,14 @@ class Calculator {
 	* @returns Returns exchange rates modification time.
 	*/
 	time_t getExchangeRatesTime();
-	/** Download current exchange rates from the Internet to local disc.
-	*
-	* @param timeout Timeout for donwload try (only used by wget)
-	* @param wget_args Extra arguments to pass to wget.
-	* @returns true if operation was successful.
-	*/
-	bool fetchExchangeRates(int timeout, string wget_args);
+	///Deprecated: wget arguments are not used
+	bool fetchExchangeRates(int seconds, string wget_args);
 	/** Download current exchange rates from the Internet to local disc with default wget arguments.
 	*
-	* @param timeout Timeout for donwload try (only used by wget)
+	* @param seconds Maximum time for donwload try
 	* @returns true if operation was successful.
 	*/
-	bool fetchExchangeRates(int timeout = 15);
+	bool fetchExchangeRates(int seconds = 15);
 	/** Check age of exchange rates on local disc. 
 	*
 	* @param n_days How old in days exchange rates may be before exchange rates need updating
@@ -1031,12 +1053,12 @@ class Calculator {
 	* @returns true if gnuplot was found.
 	*/
 	bool canPlot();
-	MathStructure expressionToPlotVector(string expression, const MathStructure &min, const MathStructure &max, int steps, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options);
-	MathStructure expressionToPlotVector(string expression, float min, float max, int steps, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options);
-	MathStructure expressionToPlotVector(string expression, const MathStructure &min, const MathStructure &max, const MathStructure &step, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options);
-	MathStructure expressionToPlotVector(string expression, float min, float max, float step, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options);
-	MathStructure expressionToPlotVector(string expression, const MathStructure &x_vector, string x_var = "\\x", const ParseOptions &po = default_parse_options);
-	bool plotVectors(PlotParameters *param, const vector<MathStructure> &y_vectors, const vector<MathStructure> &x_vectors, vector<PlotDataParameters*> &pdps, bool persistent = false);
+	MathStructure expressionToPlotVector(string expression, const MathStructure &min, const MathStructure &max, int steps, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	MathStructure expressionToPlotVector(string expression, float min, float max, int steps, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	MathStructure expressionToPlotVector(string expression, const MathStructure &min, const MathStructure &max, const MathStructure &step, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	MathStructure expressionToPlotVector(string expression, float min, float max, float step, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	MathStructure expressionToPlotVector(string expression, const MathStructure &x_vector, string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	bool plotVectors(PlotParameters *param, const vector<MathStructure> &y_vectors, const vector<MathStructure> &x_vectors, vector<PlotDataParameters*> &pdps, bool persistent = false, int msecs = 5000);
 	bool invokeGnuplot(string commands, string commandline_extra = "", bool persistent = false);
 	bool closeGnuplot();
 	bool gnuplotOpen();
@@ -1117,4 +1139,35 @@ class Calculator {
 		
 };
 
+class QalculateDate {
+	protected:
+		long int i_year;
+		long int i_month;
+		long int i_day;
+	public:
+		QalculateDate();
+		QalculateDate(long int initialyear, int initialmonth, int initialday);
+		QalculateDate(long int initialtimestamp);
+		QalculateDate(string date_string);
+		void setToCurrentDate();
+		bool set(long int newyear, int newmonth, int newday);
+		bool set(long int newtimestamp);
+		bool set(string date_string);
+		string toISOString() const;
+		string toLocalString() const;
+		long int year() const;
+		long int month() const;
+		long int day() const;
+		bool addDays(long int days);
+		bool addMonths(long int months);
+		bool addYears(long int years);
+		int weekday() const;
+		int week(bool start_sunday = false) const;
+		int yearday() const;
+		Number timestamp() const;
+		Number daysTo(const QalculateDate &date, int basis = 1, bool date_func = true) const;
+		Number yearsTo(const QalculateDate &date, int basis = 1, bool date_func = true) const;
+};
+
+
 #endif
